@@ -67,8 +67,8 @@ class Renderer(JsRenderer, JsCacheRenderer):
     file_type = '.json'
     hide_dashboard_items = True  # Used to prevent any dashboard items being rendered from main html dashboard.
 
-    def __init__(self, plugin):
-        super(JsRenderer, self).__init__(plugin)
+    def __init__(self, front_end):
+        JsRenderer.__init__(self, front_end)
         JsCacheRenderer.__init__(self)
         jsgen.register_converter(self.py2js_converter)
 
@@ -158,6 +158,8 @@ class Renderer(JsRenderer, JsCacheRenderer):
         if hasattr(v, "actor"):
             # reference to actor data for slave-grids
             result.update(obj2dict(v.actor, "actor_id"))  # to get siteDate layout index
+            # if str(v.actor) == "system.Dashboard":
+            #     print("20210617c", v.value, result)
 
         # if hasattr(v, "get_field_options"):
         if issubclass(v.__class__, FieldElement):
@@ -174,7 +176,7 @@ class Renderer(JsRenderer, JsCacheRenderer):
     def get_request_url(self, ar, *args, **kw):
         """Used for turn requests into urls"""
         if ar.actor.__name__ == "Main":
-            return self.plugin.build_plain_url(*args, **kw)
+            return self.front_end.build_plain_url(*args, **kw)
 
         st = ar.get_status()
         kw.update(st['base_params'])
@@ -191,7 +193,7 @@ class Renderer(JsRenderer, JsCacheRenderer):
             kw.setdefault(ext_requests.URL_PARAM_SORT, sc)
         # ~ print '20120901 TODO get_request_url
 
-        return self.plugin.build_plain_url(
+        return self.front_end.build_plain_url(
             ar.actor.app_label, ar.actor.__name__, *args, **kw)
 
     # from extrenderer
@@ -523,27 +525,30 @@ class Renderer(JsRenderer, JsCacheRenderer):
                       # [py2js(b) for b in v.actions.items()]
                       )
         ah = v.get_handle()
-        # grids
-        if hasattr(ah, "get_columns"):
-            columns = []
-            index_mod = 0
-            col_elems = ah.get_columns()
-            for col in col_elems:
-                d = self.elem2json(col)
-                d.update(fields_index = find(ah.store.list_fields, col.field.name,
-                                      key=lambda f: f.name) + index_mod)
-                if isinstance(col, ComboFieldElement) and not isinstance(col, SimpleRemoteComboFieldElement):
-                    # Skip the data value for multi value columns, such as choices and FK fields.
-                    # use c.fields_index -1 for data value
-                    d.update(fields_index_hidden = d['fields_index'] + 1)
-                    index_mod += 1
-                columns.append(d)
-            result['col'] = columns
-            # (dict(main=self.elem2json(col_elems.main),
-            #         col=columns,
-            #        window_size=col_layout.layout.window_size))
+        if ah.store is None:
+            settings.SITE.logger.warning("%s has no store", v)
+        else:
+            # grids
+            if hasattr(ah, "get_columns"):
+                columns = []
+                index_mod = 0
+                col_elems = ah.get_columns()
+                for col in col_elems:
+                    d = self.elem2json(col)
+                    d.update(fields_index = find(ah.store.list_fields, col.field.name,
+                                          key=lambda f: f.name) + index_mod)
+                    if isinstance(col, ComboFieldElement) and not isinstance(col, SimpleRemoteComboFieldElement):
+                        # Skip the data value for multi value columns, such as choices and FK fields.
+                        # use c.fields_index -1 for data value
+                        d.update(fields_index_hidden = d['fields_index'] + 1)
+                        index_mod += 1
+                    columns.append(d)
+                result['col'] = columns
+                # (dict(main=self.elem2json(col_elems.main),
+                #         col=columns,
+                #        window_size=col_layout.layout.window_size))
 
-        result.update(obj2dict(ah.store, "pk_index"))  # Data index which is the PK
+            result.update(obj2dict(ah.store, "pk_index"))  # Data index which is the PK
         result.update(obj2dict(v, "preview_limit "  # number of rows to render # if 0 no paginator.
                                   "use_detail_param_panel "  # show PV panel in detail
                                   "use_detail_params_value "  # in grid, use parrent PV values
@@ -567,6 +572,10 @@ class Renderer(JsRenderer, JsCacheRenderer):
             if wl is not None:
                 result.update(window_layout=wl._formpanel_name)
 
+        hotkeys = v.get_actions_hotkeys()
+        if hotkeys:
+            result.update(hotkeys=hotkeys)
+
         card_layout = getattr(v, "card_layout", None)
         if card_layout is not None:
             result.update(card_layout=card_layout._formpanel_name)
