@@ -41,6 +41,7 @@ from lino.core.views import choices_response
 from lino.core.utils import navinfo
 from lino.core.actions import ShowEmptyTable
 from lino.core import kernel
+from lino.core.store import get_atomizer
 
 from lino.modlib.users.utils import get_user_profile, with_user_profile
 from lino.api import rt
@@ -241,6 +242,31 @@ class Choices(View):
         return choices_response(rpt, request, qs, row2dict, emptyValue, field=field)
 
 
+class DelayedValue(View):
+    def get(self, request, app_label=None, actor=None, pk=None, field=None, **kw):
+        ar = action_request(
+            app_label, actor, request, request.GET, True,
+            renderer=settings.SITE.plugins.react.renderer)
+        # print("20210619a", ar)
+        # rpt = requested_actor(app_label, actor)
+        rpt = ar.actor
+        de = rpt.get_data_elem(field)
+        if de is None:
+            v = "{} has no data element {}".format(rpt, field)
+        else:
+            if pk == '-99998':
+                row = None
+            else:
+                ar.set_selected_pks(pk)
+                row = ar.selected_rows[0]
+                # raise NotImplementedError()
+            sf = get_atomizer(rpt, de, field)
+            v = sf.full_value_from_object(row, ar)
+            # v = de._lino_atomizer.full_value_from_object(row, ar)
+        # print("20210619b", de, v)
+        return json_response({'data': v})
+
+
 # Also coppied from extjs.views line for line
 class ActionParamChoices(View):
     # Examples: `welfare.pcsw.CreateCoachingVisit`
